unit Prometheus_Vid;

{$MODE Delphi}

interface

uses
    gl,
    glu,
    glut,
    glext,
    
    png,
    zlib,
    
    sdl,
    sdl_ttf,
    sdl_image,
    sdl_gfx;

type
    Vertex = array [1..2] of Int64;

{$Include Src/Video/Constant.pas}

type
	CoOrd = Packed Record
		X,Y: Int64;
	end;
    Colour = Object
	    Sdl: tSdl_Color;
	    
	    procedure SetRGBA(R,G,B,A: Int64);
	end;
    Image = Object
		//Surface: pSdl_Surface;
	    Texture: GlUint;
	    Texture_Format: Glenum;
	    NOfColors: Glint;
	    XScale, YScale: Real;
	    GlWidth, GlHeight: Integer;
	    PositionX, PositionY: Int64;
	    Rotation: Int64;
	    
	    procedure Load(Source: String);
	    procedure Draw(X,Y: Int64);
	    procedure DrawSelection(sX, sY, sW, sH, dX, dY: Int64);
	    
	    //procedure xUpdateGlData(); //Highly unstable.
	    procedure LoadFromSurface(Src: pSdl_Surface); //To be tested. Suspected unstable
	    
	    procedure Rotate(Deg: Int64);
	    procedure DrawRotated(X,Y: Int64; Deg: Int64);
	    procedure Resize(X,Y: Int64);
	    
	    function Width(): Int64;
	    function Height(): Int64;
	    procedure Empty();
	    
	end;
	Picture = Image; //Synonym definition. To be moved to TDef end.
	pImage = ^Image;
	pPicture = ^Picture;
	TilePositionDescriptor = packed record
		XPosition, YPosition: Int64;
	end;
	TileSet = object
		Tile: array [1..90000] of TilePositionDescriptor;
		Data: Image;
		OffsetX, OffSetY: Int64;
		SpacingX, SpacingY: Int64;
		TileWidth, TileHeight: Int64;
		Tiles: Int64;
		TilesPerRow, TilesPerColumn: Int64;
		
		procedure LoadFromImageFile(Src: String);
		procedure LoadFromImage(Dat: Image);
		procedure LoadFromDisk(Src: String);
		procedure SetOffSet(X,Y: Int64);
		procedure SetSpacing(X,Y: Int64);
		procedure SetTileSize(X,Y: Int64);
		procedure SetTileTable(X,Y: Int64);
		procedure SetTiles(Val: Int64);
		procedure Update();
		procedure Empty();
		
		procedure DrawTile(X, Y, TileID: Int64);
	end;
	Error = Record
		NameStr: String;
		Severity: QWord;
		Descriptor: QWord;
		Data: String;
		ID: Int64;
	end;
	
	Font = Object
	    Size: Int64;
	    Pointer_: Pointer;
	    Source: String;
	    
	    procedure Load(Src: String; Sze: Int64);
	    procedure Delete();
	    procedure SetSize(Sze: Int64);
	    procedure Draw(X,Y: Int64; Txt: String; Clr: Colour);
	    
	    function GetRenderedWidth(Str: String): Int64;
	    function GetRenderedHeight(Str: String): Int64;
	end;
	Rectangle = Object
	    Width, Height: Int64;
	    
	    VertexColour: array [1..2, 1..2, 1..4] of Int64;
	    DrawOffset: CoOrd;
	    
	    procedure SetDrawMode(Mode: Dword);
	    procedure SetDimensions(X, Y: Int64);
	    procedure Draw(X,Y: Int64);
	    procedure SetColour(Pos: DWord; R, G, B, A: Int64);
	end;
    Square = Object
	    Length: Int64;
	    
	    VertexColour: array [1..2, 1..2, 1..4] of Int64;
	    DrawOffSet: CoOrd;
	    
	    procedure SetDrawMode(Mode: Dword);
	    procedure SetLength(L: Int64);
	    procedure Draw(X, Y: Int64);
	    procedure SetColour(Pos: Dword; R, G, B, A: Int64);
	end;
    Prometheust = record
	    WindowWidth, WindowHeight: Int64;
	    ColourDepth: Int64;
	    Screen: pSdl_Surface;
		FullScreen: Boolean;
		
	    UpdateMode: Boolean;
	    RotationMode: LongWord;
	    
	    GlVRam: Gluint;
	    
	    AA_Roto: Int64;
	    AA_Scale: Int64;
	    
	    VRam: pSdl_Surface;
	    
	    SystemFont: Font;
		
		Error: Error;
		
		Initialized: Boolean;
	end;
	
var
    PrometheusVideo: Prometheust;

procedure PrometheusVid_Start();

procedure UpdateCanvas();
procedure UpdateWindow();

procedure SetWindowSize(X,Y: Int64);
procedure SetColourDepth(Val: Int64);

procedure SetPrometheusUpdateMode(Val: Boolean);
function GetPrometheusUpdateMode(): Boolean;

procedure EnableAbsoluteRotations(Val: Boolean);
procedure EnableIncrementalRotations(Val: Boolean);
function GetRotationMode(): LongWord;

procedure SetPrometheusAA_Rotating(Val: Int64);
function GetPrometheusAA_Rotating(): Int64;

procedure SetPrometheusAA_Scaling(Val: Int64);
function GetPrometheusAA_Scaling(): Int64;

procedure SetFullScreen(Val: Boolean);
procedure CreateWindow(Width, Height, BPP: Int64);

function QuickColour(R, G, B, Alpha: Int64): Colour;

procedure ClearCanvas();

procedure ReportError(NmStr: String; Sev: QWord; Desc: QWord; Dat: String);
function GetLastError(): Error;
function GetLastErrorID(): Int64;
procedure WriteErrorReport();

implementation

{$Include Src/Video/LocalAssistant.pas} //a quick file to speed things up (strtoint, inttostr and etc...)

{$Include Src/Video/Prometheus.pas}
{$Include Src/Video/Error.pas}
{$Include Src/Video/Colour.pas}
{$Include Src/Video/Primitive.pas}
{$Include Src/Video/Image.pas}
{$Include Src/Video/TileSets.pas}
{$Include Src/Video/Font.pas}

end.
